/**
 * @file	cf_stack.c
 * @author	myusgun <myusgun@gmail.com>
 */
#include "cf_stack.h"
#include "cf_list.h"
#include "cf_error.h"

#include <stdio.h>

int
CF_Stack_CreateCtx (CF_Stack_Ctx * ctx)
{
	return CF_List_CreateCtx ((CF_List_Ctx *) ctx);
}

int
CF_Stack_DestroyCtx (CF_Stack_Ctx ctx)
{
	return CF_List_DestroyCtx (ctx);
}

int
CF_Stack_Push (CF_Stack_Ctx	ctx,
			   const void	* element)
{
	int				result = 0;
	CF_List_Ctx		list = (CF_List_Ctx) ctx;
	CF_Traverser	traverser = NULL;

	result = CF_List_Front (list, &traverser);
	if (result < 0)
		return result;

	return CF_List_Insert (list, traverser, CF_DIRECTION_BEFORE, element);
}

int
CF_Stack_Pop (CF_Stack_Ctx	ctx,
			  void			** element)
{
	int			result = 0;
	CF_List_Ctx	list = (CF_List_Ctx) ctx;
	CF_Traverser	traverser = NULL;

	result = CF_Stack_Top (ctx, element);
	if (result < 0)
		return result;

	result = CF_List_Front (list, &traverser);
	if (result < 0)
		return result;

	return CF_List_Remove (ctx, &traverser);
}

int
CF_Stack_Top (CF_Stack_Ctx 	ctx,
			  void			** element)
{
	int				result = 0;
	CF_List_Ctx		list = (CF_List_Ctx) ctx;
	CF_Traverser	traverser = NULL;

	result = CF_List_Front (list, &traverser);
	if (result < 0)
		return result;

	return CF_List_GetElement (traverser, element);
}

int
CF_Stack_GetSize (CF_Stack_Ctx ctx)
{
	return CF_List_GetSize ((CF_List_Ctx) ctx);
}
